#encoding: UTF-8 
=begin
/***************************************************************************
 *   ©2018-2019 Michael Uplawski <michael.uplawski@uplawski.eu>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
=end

require_relative 'argparser'
require_relative 'table_template'
require_relative 'styles_template'
require_relative 'logging'
require_relative 'user_input'
require_relative 'file_checking'
require 'date'
require 'tmpdir'
require 'fileutils'

# The main application class
class Generator
  self.extend(Logging)
  @@log = init_logger(STDOUT)
  include File_Checking
  def initialize(args)
    @log = @@log
    
    @options = ArgParser.parse(args)
    @log.debug('options: ' << @options.inspect)

    generate
  end

  private
  def generate
    sequence = @options.date ? date_sequence : number_sequence 
    @log.debug('sequence is ' << sequence.inspect)
    table = $TABLE_TEMPLATE.dup  
    rows = ''
    sequence.each do |ele|
      rows << '<w:tr><w:tc><w:p><w:r><w:t>'
      rows << format(ele) << '</w:t></w:r></w:p></w:tc>'
      (@options.cols - 1).times { rows << '<w:tc></w:tc>' }
      rows << '</w:tr>'
    end
    @log.debug('rows is ' << rows.inspect)
    table.sub!('<w:tr></w:tr>', rows.to_s)
    if !@options.file
      @log.info("document.xml:\n" << table.inspect)
    elsif File.exist?(@options.file)
      @log.warn(@options.file.dup << " exists !\nShould the file be replaced?")
      if wait_for_user.chr.downcase != 'y'
        @log.info('okay, doing nothing')
        exit true
      end
    end
    word = File.join(Dir::tmpdir, 'word')
    doc_file = File.join(word, 'document.xml')
    style_file = File.join(word, 'styles.xml')
    File.delete(doc_file) if File.exist?(doc_file)
    File.delete(style_file) if File.exist?(style_file)
    Dir.rmdir(word) if Dir.exist?(word)
    Dir.mkdir(word) 

    #File.write(doc_file, table )
    #File.write(style_file, $STYLES_TEMPLATE )

    File.write(doc_file, table.gsub(/\n\s*</,'<' ) )
    File.write(style_file, $STYLES_TEMPLATE.gsub(/\n\s*</,'<') )
    Dir::chdir(Dir::tmpdir)

    exec('zip', '-9r', @options.file, File.basename(word) )
    if File.exist?(@options.file) 
      @log.info(@options.file << "has been written")
    else
      @log.error(@options.file << "could NOT be written.")
    end
  end

  def format(ele)
    ele.respond_to?(:to_date) ? ele.strftime(@options.format) : sprintf("%" << @options.n_width.to_s << "d", ele)
  end

  # creates a sequence of dates
  # TODO: Exclusions
  def date_sequence
    sd = Date.parse(@options.s_value)
    ed = Date.parse(@options.e_value)
    seq = Array.new
    sd.upto(ed) {|dt| seq << dt}
    seq
  end

  # creates a sequence of numbers
  def number_sequence
    seq = Array.new
    @options.s_value.upto(@options.e_value) {|n| seq << n}
    seq
  end
end
